/**
 * @(#)Score.java
 *
 * Created on July 26, 2000, 1:08 PM
 */


/**
 *
 * @author  furyhawk <lookmeow@magix.com.sg>
 * @author  Raimar Falke <rf13@inf.tu-dresden.de>
 * @version 0.14
 */


/**
   ********************************************************************
 CivLog - Copyleft (C) 2000 - Wong Teck Meng
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
**********************************************************************
                                                                      */

package org.freeciv.civlog.data;

import org.freeciv.civlog.Debug;
import org.freeciv.civlog.util.MaxMin;
import org.freeciv.civlog.FreecivConstants;

import java.util.*;
import java.io.*;

import javax.swing.*;
import java.awt.Container;

public class Score extends Object
{
    private boolean isValid;
    /**
     * Mappes from EntryKey to ScoreEntry.
     */
    private Hashtable civscore;
    private Hashtable turn2year_;
    private Hashtable year2turn_;

    public String playerNames[];

    /** Records number of players in this score(file) */
    public int numofPlayers = 0;

    /** Cached maximum values for the different types of data. It is
     * valid after the first call to loadScoreFile */
    public int maxValues[];
    public int startYear, endYear;
    public int firstTurn, lastTurn;

    /** JFrame for Message Dialog */
    private JFrame frame;
    private Container container;

    /** Creates new Score */
    public Score (JFrame pframe,Container container)
    {
        Debug.trace ("Score.Score(): initializing" );

        /** assign parent frame */
        frame = pframe;
        this.container=container;
        isValid=false;
        Debug.trace ( "Score.Score(): done" );
    }

    public int getNumPlayers ()
    {
        if(!isValid)
            throw new InternalError();
        return numofPlayers;
    }

    /** don't be fool by the get, it set data also */
    public ScoreEntry getEntry (int year, int player)
    {
        if(!isValid)
            throw new InternalError();

        return getEntry0(year,player);
    }

    private ScoreEntry getEntry0 (int year, int player)
    {
        ScoreEntry result = findEntry0 (year,player);

        if (result==null)
        {
            /** nothing found -> create new one */
            result = new ScoreEntry (year, player);
            EntryKey key = new EntryKey(year,player);
            civscore.put (key,result);
        }

        return result;
    }

    public ScoreEntry findEntry (int year, int player)
    {
        if(!isValid)
            throw new InternalError();

        return findEntry0(year,player);
    }

    private ScoreEntry findEntry0 (int year, int player)
    {
        EntryKey key = new EntryKey (year,player);
        return (ScoreEntry)civscore.get (key);
    }

    /** calculate max-Values dynamically depending on values between 2
     * years used byte CivLogDraw.rescale() this is terribly
     * inefficient but usable on my computer 1 sec wait on my box
     * though (AMD K6-III 400 -lower to 333 sometimes because of local
     * tropical climate) */
    public MaxMin getMaxMinValue(int dataType, int startYear, int endYear,
                                 boolean playerSelected[])
    {
        if(!isValid)
            throw new InternalError();

        if(numofPlayers!=playerSelected.length)
            throw new InternalError();

        int value = 0;

        MaxMin values = new MaxMin();

        for (int year=startYear; year<endYear; year++)
        {
            /** invalid year, skip it */
            if (findEntry (year,0) == null)
                continue;

            for (int player=0; player < numofPlayers; player++)
            {
                /** localized the search, only selected players Max is
                    parsed */
                if (!playerSelected[player]) continue;

                ScoreEntry current = findEntry (year, player);
                value = current.getData (dataType);
                /** get selected data */
                values.max = Math.max(values.max, value);
                values.min = Math.min(values.min, value);
            }
        }
        return values;
    }

    /** return load success or failure */
    public boolean loadScoreFile(String filePath)
    {
        Debug.trace ( "Score.load: reading all lines......." );

        String lines[];
        try
        {
            File theFile = new File(filePath);
            BufferedReader in = new BufferedReader (new FileReader (theFile));
            Vector linesVector = new Vector();
            String line;
            for(;;)
            {
                line = in.readLine ();
                if(line==null)
                    break;
                linesVector.addElement(line);
            }
            in.close ();
            lines = new String[linesVector.size()];
            linesVector.copyInto(lines);
        }
        catch (IOException e)
        {
            e.printStackTrace();
            return false;
        }

        int currentLine = 0;

        ProgressMonitor progressMonitor = new
                                          ProgressMonitor(container, "loading score log", "wait", 0,
                                                          lines.length);
        progressMonitor.setProgress(currentLine);
        progressMonitor.setMillisToDecideToPopup(0);
        progressMonitor.setMillisToPopup(0);
        container.show();

        if(currentLine < lines.length)
        {
            Debug.trace ( "Score.load: parsing for "+
                          "FREECIV header" );
            if (lines[currentLine].startsWith ("#FREECIV SCORELOG"))
            {
                /** TODO: button change to loadme */
                //JOptionPane.showMessageDialog (frame,
                //"It is a Civlog File: " + filePath
                //+ "\nLoading.....Wait even after pressing OK"
                //);

                Debug.trace ( "Score.load: Loading Freeciv scorefile: "
                              + filePath );
            }
            else
            {
                JOptionPane.showMessageDialog(frame,"Not a Civlog File: "+
                                              filePath, "Error",
                                              JOptionPane.ERROR_MESSAGE );
                Debug.trace ( "ERROR: Score.load: "+
                              "Not a Civlog File: " + filePath );
                /** quit if no a civlog file */
                return (false);
            }
        }
        else
            throw new InternalError();

        /** FIXME: add more checks */
        /** check for header and get rid to it */
        Debug.trace ( "Score.load: parsing data type header" );
        for (;;)
        {
            currentLine++;
            progressMonitor.setProgress(currentLine);
            if(currentLine >= lines.length)
            {
                JOptionPane.showMessageDialog (frame, "Unexpected end "+
                                               "of file while parsing "+
                                               "data type header",
                                               "Error",
                                               JOptionPane.ERROR_MESSAGE );
                Debug.trace ( "ERROR: Score.load: parsing "+
                              "type header failed" );
                return (false);
            }

            StringTokenizer st = new StringTokenizer (lines[currentLine]);
            if (st.countTokens () > 2)
            {
                /** shouldn't be */
                JOptionPane.showMessageDialog (frame, "Invalid format "+
                                               "in line '"+lines[currentLine]+
                                               "': expected 2 tokens",
                                               "Error",
                                               JOptionPane.ERROR_MESSAGE );
                Debug.trace ( "ERROR: Score.load: "+
                              "parsing type header II" );
                return (false);
            }

            if(st.countTokens()==1)
            {
                if(st.nextToken ().equalsIgnoreCase ("end") )
                {
                    /** every OK! goto next parse*/
                    break;
                }
                else
                    throw new Error("Invalid format");
            }
            st.nextToken();
            String abc=st.nextToken();

            if(!FreecivConstants.isValidTypeName(abc))
            {
                JOptionPane.showMessageDialog (frame,
                                               "Unknown type of data",
                                               "Error",
                                               JOptionPane.ERROR_MESSAGE );
                Debug.trace ( "ERROR: Score.load: "+
                              "parsing data type header III" );
                return (false);
            }
        }

        /** read in the lines and get no. of players */
        Debug.trace ( "Score.load: parsing players header at line "+
                      currentLine);

        int firstPlayerLine=currentLine+1;
        for (;;)
        {
            currentLine++;
            progressMonitor.setProgress(currentLine);
            if(currentLine >= lines.length)
            {
                JOptionPane.showMessageDialog (frame,
                                               "Error while players "+
                                               "header is parsed",
                                               "Error",
                                               JOptionPane.ERROR_MESSAGE );
                Debug.trace ( "ERROR: Score.load: "+
                              "parsing players header" );
                return (false);
            }

            /** check for end of players header */

            if (lines[currentLine].startsWith ("end"))
            {
                /** everything OK! goto next parse */
                break;
            }
        }

        numofPlayers = currentLine-firstPlayerLine;
        playerNames = new String[numofPlayers];

        for (int i=firstPlayerLine; i<currentLine; i++)
        {
            String line = lines[i];
            int index = line.indexOf (" ");
            String playerNumber = line.substring (0,index);
            String playerName = line.substring (index);
            int playerNumberAsInt;

            try
            {
                playerNumberAsInt = Integer.parseInt (playerNumber);
            }
            catch (NumberFormatException nfe)
            {
                throw new RuntimeException ("Invalid format: "+
                                            playerNumber);
            }

            playerNames[playerNumberAsInt] = playerName;
        }

        Debug.trace ( "Score.load: found "+numofPlayers+" players");

        civscore = new Hashtable();
        maxValues=new int[FreecivConstants.NUMBER_OF_DATA_TYPES];

        /** FIXME: add more checks and optimize*/

        /* initial condition, use for setting start year */
        boolean firstPass = true;
        int lastYear=Integer.MAX_VALUE;
        int turn=0;
        turn2year_=new Hashtable();
        year2turn_=new Hashtable();

        Debug.trace ( "Score.load: parsing main data at line "+currentLine);
        for (;;)
        {
            currentLine++;
            progressMonitor.setProgress(currentLine);
            if(currentLine >= lines.length)
                break;

            /* can this part be optimized?  It is allocating too
             * much memory? */
            StringTokenizer st = new StringTokenizer (lines[currentLine]);

            if (st.countTokens () < 4)
            {
                throw new RuntimeException ("Not enough fields");
            }

            String typeStr = st.nextToken ();
            String playerStr = st.nextToken ();
            String yearStr = st.nextToken ();
            String valueStr = st.nextToken ();
            int type, player, year;
            int value;

            /** area value range can be very big */
            try
            {
                type = Integer.parseInt (typeStr);
                player = Integer.parseInt (playerStr);
                year = Integer.parseInt (yearStr);
                value = Integer.parseInt (valueStr);
            }
            catch (NumberFormatException nfe)
            {
                throw new RuntimeException ("Illegal line " +
                                            lines[currentLine] + nfe);
            }

            maxValues[type] = Math.max(maxValues[type],value);

            if (firstPass)
            {
                startYear = year;
                firstPass = false;
                lastYear=year-1;
            }
            endYear = year;

            if(year!=lastYear)
            {
                //Debug.trace("setting year="+year+" = "+turn+"=turn");
                turn2year_.put(new Integer(turn),new Integer(year));
                year2turn_.put(new Integer(year),new Integer(turn));
                turn++;
            }
            lastYear=year;

            ScoreEntry clde = getEntry0 (year, player);
            clde.setData (type, value);
        }
        progressMonitor.close ();

        JOptionPane.showMessageDialog (frame, "No. of players: " +
                                       numofPlayers + "\nStart Year: " +
                                       startYear + "\nEnd Year: " +
                                       endYear +"\nTurns: "+
                                       (year2turn(endYear)-
                                        year2turn(startYear)));

        Debug.trace ( "Score.load: Start Year: " + startYear +
                      " End Year: " + endYear );

        Debug.trace ( "Score.load: returning successful" );
        isValid=true;
        firstTurn=year2turn(startYear);
        lastTurn=year2turn(endYear);
        return (true);
    }

    public boolean isValid()
    {
        return isValid;
    }

    public int turn2year(int turn)
    {
        Integer i=(Integer)turn2year_.get(new Integer(turn));
        if(i==null)
            throw new InternalError("can't find turn "+turn+" in "+turn2year_);
        return i.intValue();
    }

    public int year2turn(int year)
    {
        Integer i=(Integer)year2turn_.get(new Integer(year));
        if(i==null)
            throw new InternalError("can't find year "+year+" in "+year2turn_);
        return i.intValue();
    }

    public int getNearestKnownYear(int year)
    {
        if(!isValid)
            throw new InternalError();
        if(year<startYear || year>endYear)
            throw new InternalError();
        for(int i=0;i<50;i++)
        {
            if(year2turn_.containsKey(new Integer(year+i)))
                return year+i;
            if(year2turn_.containsKey(new Integer(year-i)))
                return year-i;
        }
        throw new InternalError("Can't find nearest year; starting from "+
                                year+" in "+year2turn_);
    }
}
// End of Score.java
