/**
 * @(#)Graph.java - responsible for drawing a graph
 *
 * Created on July 23, 2000, 4:37 PM
 */


/**
 *
 * @author  furyhawk <lookmeow@magix.com.sg>
 * @author  Raimar Falke <rf13@inf.tu-dresden.de>
 * @author  Michael Grundel <michael42@grundel.de>
 * @version 0.14
 */

/**
   ********************************************************************
 CivLog - Copyleft (C) 2000 - Wong Teck Meng
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
**********************************************************************
                                                                      */

package org.freeciv.civlog.gui;

import org.freeciv.civlog.Debug;
import org.freeciv.civlog.CLProperties;
import org.freeciv.civlog.FreecivConstants;
import org.freeciv.civlog.ColorManager;
import org.freeciv.civlog.data.Score;
import org.freeciv.civlog.data.ScoreEntry;
import org.freeciv.civlog.util.PolyLine;
import org.freeciv.civlog.util.PolyLineGroup;
import org.freeciv.civlog.util.MaxMin;

import java.awt.*;
import java.awt.event.*;

import java.util.*;

import javax.swing.*;

public class Graph
            extends javax.swing.JPanel
            implements ViewChangeListener
{
    private Score score;
    private View view;

    /** The data type like FreecivConstants.POP and similar */
    private int myDataType;

    private static final boolean showColors=false;
    private boolean draggingValid=false, draggingActive=false;
    private Point draggingStart, draggingCurrent;
    private PolyLineGroup lines;

    /** Creates new form Graph */
    public Graph (Score data, View view, int myDataType)
    {
        this.score = data;
        this.view = view;
        initComponents ();
        this.myDataType = myDataType;

        setBackground(ColorManager.getGraphBackgroundColor());
        view.addViewChangeListener(this);

        addMouseMotionListener(new MouseMotionListener()
                               {
                                   public void mouseDragged(MouseEvent e)
                                   {
                                       graphMouseDragged(e);
                                   }

                                   public void mouseMoved(MouseEvent e)
                                   {
                                       //graphMouseMoved(e);
                                   }
                               }
                              );

        addMouseListener(new MouseListener()
                         {
                             public void mouseClicked(MouseEvent e)
                             {
                                 graphMouseClicked(e);
                             }
                             public void mousePressed(MouseEvent e)
                             {
                                 graphMousePressed(e);
                             }
                             public void mouseReleased(MouseEvent e)
                             {
                                 graphMouseReleased(e);
                             }
                             public void mouseEntered(MouseEvent e)
                             {
                                 // graphMouseEntered(e);
                             }
                             public void mouseExited(MouseEvent e)
                             {
                                 graphMouseExited(e);
                             }
                         }
                        );
    }

    private void drawDraggingSelection()
    {
        Graphics g=getGraphics();
        g.setXORMode(Color.green);
        g.setColor(Color.red);

        int startX,endX;

        startX=Math.min(draggingStart.x,draggingCurrent.x);
        endX=Math.max(draggingStart.x,draggingCurrent.x);
        g.fillRect(startX,0,endX-startX,getSize().height);
    }

    void graphMouseDragged(MouseEvent e)
    {
        if(!draggingActive)
            return;

        drawDraggingSelection();

        draggingCurrent=e.getPoint();

        drawDraggingSelection();
    }

    void graphMouseClicked(MouseEvent e)
    {
        if((e.getModifiers()&MouseEvent.BUTTON2_MASK)==
                MouseEvent.BUTTON2_MASK &&
                !draggingActive &&
                draggingValid)
        {
            draggingValid=false;
            // zoom in
            int startRX, endRX;

            startRX=Math.min(draggingStart.x,draggingCurrent.x);
            endRX=Math.max(draggingStart.x,draggingCurrent.x);

            Debug.trace("Zoom in: pixel range "+startRX+" - "+endRX);

            float startPos[]=lines.unrender(startRX,0);
            float endPos[]=lines.unrender(endRX,0);

            Debug.trace("  unrendered: "+startPos[0]+" - "+endPos[0]);
            int startTurn,endTurn;

            startTurn=Math.round(startPos[0]);
            endTurn=Math.round(endPos[0]);

            if(view.isDisplayLabeledInYears())
            {
                if(startTurn<score.startYear)
                    startTurn=score.startYear;

                if(endTurn>score.endYear)
                    endTurn=score.endYear;

                startTurn=score.year2turn(score.
                                          getNearestKnownYear(startTurn));
                endTurn=score.year2turn(score.
                                        getNearestKnownYear(endTurn));
            }
            else
            {
                if(startTurn<score.firstTurn)
                    startTurn=score.firstTurn;
                if(endTurn>score.lastTurn)
                    endTurn=score.lastTurn;
            }
            if(endTurn==startTurn)
                endTurn++;
            view.setStartTurn(startTurn);
            view.setEndTurn(endTurn);
            return;
        }
        if((e.getModifiers()&MouseEvent.BUTTON3_MASK)==
                MouseEvent.BUTTON3_MASK &&
                !draggingActive)
        {
            // ignore right click if zoom out already
            if ( view.getStartTurn() == score.firstTurn
                 && view.getEndTurn() == score.lastTurn )
                return;

            draggingValid=false;
            // zoom out
            view.setStartTurn(score.firstTurn);
            view.setEndTurn(score.lastTurn);
            return;
        }
    }

    void graphMousePressed(MouseEvent e)
    {
        if((e.getModifiers()&MouseEvent.BUTTON1_MASK)!=MouseEvent.BUTTON1_MASK)
            return;

        // user dragging to make a selection
        if(draggingValid)
        {
            drawDraggingSelection();
        }
        draggingValid=false;
        draggingActive=true;
        draggingStart=e.getPoint();
        draggingCurrent=e.getPoint();
    }

    void graphMouseReleased(MouseEvent e)
    {
        if(!draggingActive)
            return;

        draggingValid=true;
        draggingActive=false;
        setToolTipText ("Middle Mouse Button to zoom in");
    }

    void graphMouseExited(MouseEvent e)
    {
        if(draggingActive || draggingValid)
            drawDraggingSelection();
        draggingActive=false;
        draggingValid=false;
        setToolTipText ("Right Mouse Button to zoom out");
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the FormEditor.
     */
    private void initComponents ()//GEN-BEGIN:initComponents
    {
        setLayout (new java.awt.BorderLayout ());
        setPreferredSize (new java.awt.Dimension (500, 100));
        setBorder (new javax.swing.border.TitledBorder (""));
        setMinimumSize (new java.awt.Dimension (500, 100));

    }//GEN-END:initComponents

    public void paint (Graphics g)
    {
        super.paint(g);

        /** check if any data is loaded */
        if ( view.getNumOfPlayers()==0 || !score.isValid() ||
                view.getNumOfPlayersSelected()==0)
        {
            Debug.trace ( "Graph.paint(): no permission or data empty: "
                          +" numofPlayers: "+score.numofPlayers
                          +" myDataType: "+myDataType
                          +" numOfPlayerSelected: "+
                          view.getNumOfPlayersSelected());
        }
        else
        {
            drawGraph (g);
        }

        if(showColors)
        {
            for(int i=0;i<30;i++)
            {
                int height=20;
                g.setColor ( Color.white); // COOKIE::Color
                g.fillRect(0,i*height,100,(i+1)*height);

                g.setColor(ColorManager.getColorForPlayer(i));
                g.drawString(i+" color ="+ColorManager.getColorForPlayer(i),
                             10,i*height+height);
            }
        }
    }/* end - paint() */

    public void drawGraph (Graphics g)
    {
        boolean perCity=view.isPerCity(myDataType);
        boolean percental=view.isPercental(myDataType);

        Font font = g.getFont ();
        font = font.deriveFont ( 9f);
        g.setFont (font);

        Dimension d=getSize();
        setBackground(ColorManager.getGraphBackgroundColor());

        String tmp=FreecivConstants.DATA_DESCRIPTION[myDataType];
        if(perCity && FreecivConstants.DATA_PER_CITY[myDataType])
            tmp+=" per city";

        FontMetrics fm = g.getFontMetrics();
        int maxPlayerNameWidth=0;
        for(int player=0;player<score.numofPlayers;player++)
        {
            if (!view.isPlayerSelected(player))
                continue;
            maxPlayerNameWidth=Math.max(maxPlayerNameWidth,
                                        fm.stringWidth(score.playerNames[player]));
        }
        lines=new PolyLineGroup(g,d,0,0,0,maxPlayerNameWidth+4,
                                (view.isDisplayLabeledInYears()?
                                 "time in years":
                                 "time in turns"),
                                tmp,percental);

        int indexToSelectedPlayer[]=new int[view.getNumOfPlayersSelected()];

        // for each selected player add a PolyLine
        for (int playerNo=0,index=0;
                playerNo < score.numofPlayers;
                playerNo++ )
        {
            if (!view.isPlayerSelected(playerNo))
                continue;

            indexToSelectedPlayer[index]=playerNo;
            lines.addPolyLine ( new PolyLine ());
            index++;
        }

        /**
         *  the main core for calculating the points
         */
        int startX,endX;

        startX=view.getStartTurn();
        endX=view.getEndTurn();

        if(view.isDisplayLabeledInYears())
        {
            startX=score.turn2year(startX);
            endX=score.turn2year(endX);
        }

        for (int x=startX; x<endX; x++)
        {
            int year;

            if(view.isDisplayLabeledInYears())
                year=x;
            else
                year=score.turn2year(x);

            /** invalid year, skip it */
            if (score.findEntry (year, 0) == null)
                continue;

            for (int index=0; index<indexToSelectedPlayer.length;index++)
            {
                int player=indexToSelectedPlayer[index];
                ScoreEntry current = score.findEntry (year, player);

                float y=current.getData (myDataType);
                if(perCity && FreecivConstants.DATA_PER_CITY[myDataType])
                {
                    int cities=current.getData (FreecivConstants.CITIES);
                    if(cities>0)
                        y/=(float)cities;
                }
                PolyLine theLine = lines.getLine(index);
                theLine.addPoint ((float)x, y);
            }/* end - inner for (index) */
        }/* end - outer for (year) */

        /** Assign Players Names to the label */
        for (int index=0; index<indexToSelectedPlayer.length;index++)
        {
            int player=indexToSelectedPlayer[index];

            PolyLine theLine = lines.getLine(index);
            theLine.setLabel ( (String)score.playerNames[player]);
            theLine.setColor ( ColorManager.getColorForPlayer(player));
        }

        /** update the setting to PolyLineGroup */
        lines.setShowDashGrid(view.isDisplayDashGrid());

        /** initialized the PolyLineGroup internal variables first */
        lines.renderGroup();

        if(!percental && view.isDisplayGrid())
            lines.drawGrid();

        for (int index=0; index<indexToSelectedPlayer.length;index++)
        {
            PolyLine theLine = lines.getLine(index);

            theLine.drawPolyLine ();
        }

        if(percental && view.isDisplayGrid())
            lines.drawGrid();

        for (int index=0; index<indexToSelectedPlayer.length;index++)
        {
            PolyLine theLine = lines.getLine(index);

            theLine.drawLabel ();
            theLine.drawMaxPoint ();

            if (view.getNumOfPlayersSelected() == 1)
            {
                theLine.drawAverage ();
                theLine.drawMinPoint ();
            }
        }
    }/* end - drawGraph() */

    public void viewChanged(ViewChangeEvent evt)
    {
        repaint();
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
}
// End of Graph.java
