/**
 * @(#)RangeMonitor.java - The top bar use for displaying the selected
 *                         and total year/turn range.
 *
 * Created on September 12, 2000, 9:05 PM
 */


/**
 *
 * @author  Raimar Falke <rf13@inf.tu-dresden.de>
 * @version 0.01
 */

/**
   ********************************************************************
 CivLog - Copyleft (C) 2000 - Wong Teck Meng
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
**********************************************************************
                                                                      */

package org.freeciv.civlog.gui;

import org.freeciv.civlog.Debug;
import org.freeciv.civlog.ColorManager;
import org.freeciv.civlog.data.Score;

import java.awt.*;

import java.util.*;

import javax.swing.*;

public class RangeMonitor
            extends JPanel
            implements ViewChangeListener
{
    private Score score=null;
    private View view=null;

    public RangeMonitor()
    {
        setBackground(Color.white);
    }

    public void init(Score score, View view)
    {
        this.score=score;
        this.view=view;
        view.addViewChangeListener(this);
    }

    public void paint(Graphics g)
    {
        g.setColor(getBackground());
        g.fillRect(0,0,getSize().width,getSize().height);

        if(view==null)
            return;

        Rectangle outerRect=new Rectangle(30,5,getSize().width-60,
                                          getSize().height-25);
        float startPercent=(float)(view.getStartTurn());
        float endPercent=(float)(view.getEndTurn());
        float tmp=score.lastTurn-score.firstTurn;
        startPercent/=tmp;
        endPercent/=tmp;

        Rectangle wholeBar=new Rectangle(outerRect.x+2,outerRect.y+2,
                                         outerRect.width-4,outerRect.height-4);
        Rectangle selectedBar=new Rectangle(wholeBar.x+(int)(startPercent*
                                            wholeBar.width),
                                            wholeBar.y,
                                            (int)((endPercent-startPercent)*
                                                  (wholeBar.width)),
                                            wholeBar.height+1);
        g.setColor(Color.black);
        g.drawRect(outerRect.x,outerRect.y,outerRect.width,outerRect.height);

        g.setColor(Color.blue);
        g.fillRect(selectedBar.x,selectedBar.y,
                   selectedBar.width,selectedBar.height);
        g.setColor(Color.black);

        String startLabel, centerLabel,endLabel;
        if(view.isDisplayLabeledInYears())
        {
            startLabel=""+score.startYear;
            endLabel=""+score.endYear;
            centerLabel=""+(score.turn2year(view.getEndTurn())-
                            score.turn2year(view.getStartTurn()))+" years";
        }
        else
        {
            startLabel=""+score.firstTurn;
            endLabel=""+score.lastTurn;
            centerLabel=""+(view.getEndTurn()-
                            view.getStartTurn())+" turns";
        }

        FontMetrics fm=g.getFontMetrics();

        g.drawString(startLabel,outerRect.x-fm.stringWidth(startLabel)/2,
                     getSize().height-3);
        g.drawString(centerLabel,outerRect.x+outerRect.width/2-
                     fm.stringWidth(centerLabel)/2,getSize().height-3);
        g.drawString(endLabel,outerRect.x+outerRect.width-
                     fm.stringWidth(endLabel)/2,getSize().height-3);
    }

    public Dimension getPreferredSize()
    {
        return getMinimumSize();
    }

    public Dimension getMinimumSize()
    {
        return new Dimension(300,40);
    }

    public void viewChanged(ViewChangeEvent evt)
    {
        if(evt.getCode()==ViewChangeEvent.C_RANGE_SELECTION ||
                evt.getCode()==ViewChangeEvent.C_OTHER)
            repaint();
    }
}
// End of RangeMonitor.java