/*
 * @(#)View.java
 *
 *
 * Created on July 28, 2000, 3:57 PM
 */


/**
 *
 * @author  Raimar Falke <rf13@inf.tu-dresden.de>
 * @version 0.01
 */

/**
   ********************************************************************
 CivLog - Copyright (C) 2000 - Wong Teck Meng
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
**********************************************************************
                                                                      */

package org.freeciv.civlog.gui;

import org.freeciv.civlog.FreecivConstants;
import org.freeciv.civlog.CLProperties;
import org.freeciv.civlog.Debug;
import java.util.Vector;
import java.util.Enumeration;
import java.util.StringTokenizer;

public class View
{
    private boolean playerSelected[];
    private int numOfPlayerSelected;
    private int numOfPlayers;
    private int startTurn,endTurn;
    private DataTypeDescriptor dataTypes[];
    private boolean displayYears;
    private Vector listeners;
    private boolean displayGrid;
    private boolean displayDashGrid;

    public View(int numOfPlayers)
    {
        listeners=new Vector();
        this.numOfPlayers=numOfPlayers;
        playerSelected=new boolean[numOfPlayers];
        numOfPlayerSelected=0;

        for (int i=0; i<numOfPlayers; i++)
            playerSelected[i] = false;

        dataTypes=new DataTypeDescriptor[FreecivConstants.
                                         NUMBER_OF_DATA_TYPES];

        for(int i=0;i<dataTypes.length;i++)
            dataTypes[i]=new DataTypeDescriptor(i);

        loadDefaults();
    }

    private void loadDefaults()
    {
        setDisplayGrid("true".equals(CLProperties.getProperty ("showGrid")));
        setDisplayDashGrid("true".equals(CLProperties.
                                         getProperty ("showDashGrid")));
        setPerCity("true".equals(CLProperties.getProperty ("perCity")));
        setPercental("true".equals(CLProperties.getProperty ("percental")));
        setDisplayLabeledInYears("true".equals(CLProperties.
                                               getProperty ("showYears")));
        String players=CLProperties.getProperty ("players");
        if(players==null)
            players="";
        StringTokenizer st=new StringTokenizer(players,",");
        while (st.hasMoreTokens())
        {
            int tmp;

            String a=st.nextToken().trim();
            if(a.length()==0)
                continue;

            try
            {
                tmp=Integer.parseInt (a);
            }
            catch (NumberFormatException nfe)
            {
                nfe.printStackTrace();
                throw new InternalError();
            }
            if(tmp<numOfPlayers)
                setPlayerSelected(tmp,true);
        }

        String dataTypesString=CLProperties.getProperty ("dataTypes");
        if(dataTypesString==null)
            dataTypesString="";
        st=new StringTokenizer(dataTypesString,",");
        while (st.hasMoreTokens())
        {
            int tmp;

            String a=st.nextToken().trim();
            if(a.length()==0)
                continue;

            try
            {
                tmp=Integer.parseInt (a);
            }
            catch (NumberFormatException nfe)
            {
                nfe.printStackTrace();
                throw new InternalError();
            }
            if(tmp<numOfPlayers)
                setDataTypeSelected(tmp,true);
        }
    }

    public boolean isDisplayGrid()
    {
        return displayGrid;
    }

    public boolean isDisplayDashGrid()
    {
        return displayDashGrid;
    }

    public void setDisplayGrid(boolean newDisplayGrid)
    {
        if(displayGrid==newDisplayGrid)
            return;

        displayGrid=newDisplayGrid;
        CLProperties.setProperty ("showGrid", ""+displayGrid);
        fireViewChangedEvent(ViewChangeEvent.C_OTHER);
    }

    public void setDisplayDashGrid(boolean newDisplayDashGrid)
    {
        if(displayDashGrid==newDisplayDashGrid)
            return;

        displayDashGrid=newDisplayDashGrid;
        CLProperties.setProperty ("showDashGrid", ""+displayDashGrid);
        fireViewChangedEvent(ViewChangeEvent.C_OTHER);
    }

    public boolean[] getPlayerSelectedArray()
    {
        boolean ret[]=new boolean[playerSelected.length];
        System.arraycopy(playerSelected,0,ret,0,playerSelected.length);
        return ret;
    }

    public int[] getPlayerSelectedIndices()
    {
        int result[]=new int[numOfPlayerSelected];
        int j=0;
        for(int i=0;i<playerSelected.length;i++)
        {
            if(playerSelected[i])
            {
                result[j]=i;
                j++;
            }
        }
        return result;
    }

    public int getNumOfPlayersSelected()
    {
        return numOfPlayerSelected;
    }

    public boolean isPlayerSelected(int player)
    {
        return playerSelected[player];
    }

    public void setPlayerSelected(int number,boolean selected)
    {
        if (playerSelected[number]==selected)
            return;

        playerSelected[number]=selected;

        if(selected)
            numOfPlayerSelected++;
        else
            numOfPlayerSelected--;
        fireViewChangedEvent(ViewChangeEvent.C_PLAYER_SELECTION,
                             number,selected);
        String tmp="";
        for(int i=0;i<playerSelected.length;i++)
            if(playerSelected[i])
                tmp+=""+i+", ";
        CLProperties.setProperty ("players", tmp);
    }

    public boolean isDataTypeSelected(int dataType)
    {
        return dataTypes[dataType].selected;
    }

    public void setDataTypeSelected(boolean selected)
    {
        boolean update=false;

        for(int i=0;i<dataTypes.length;i++)
        {
            if(dataTypes[i].selected!=selected)
            {
                dataTypes[i].selected=selected;
                fireViewChangedEvent(ViewChangeEvent.
                                     C_DATA_TYPE_SELECTION,i,selected);
                update=true;
            }
        }
        if(update)
            updateDataTypeProperty();
    }

    private void updateDataTypeProperty()
    {
        String tmp="";
        for(int i=0;i<dataTypes.length;i++)
            if(dataTypes[i].selected)
                tmp+=""+i+", ";
        CLProperties.setProperty ("dataTypes", tmp);
    }

    public void setDataTypeSelected(int dataType, boolean selected)
    {
        if (dataTypes[dataType].selected==selected)
            return;

        dataTypes[dataType].selected=selected;
        fireViewChangedEvent(ViewChangeEvent.
                             C_DATA_TYPE_SELECTION,dataType,selected);
        updateDataTypeProperty();
    }

    public boolean isPerCity(int dataType)
    {
        return dataTypes[dataType].perCity;
    }

    public void setPerCity(boolean selected)
    {
        for(int i=0;i<dataTypes.length;i++)
        {
            if(dataTypes[i].perCity!=selected)
            {
                dataTypes[i].perCity=selected;
                fireViewChangedEvent(ViewChangeEvent.C_DATA_TYPE_GENERAL,i);
            }
        }
        CLProperties.setProperty ("perCity", ""+selected);
    }

    public boolean isPercental(int dataType)
    {
        return dataTypes[dataType].percental;
    }

    public void setPercental(boolean percental)
    {
        for(int i=0;i<dataTypes.length;i++)
        {
            if(dataTypes[i].percental!=percental)
            {
                dataTypes[i].percental=percental;
                fireViewChangedEvent(ViewChangeEvent.C_DATA_TYPE_GENERAL,i);
            }
        }
        CLProperties.setProperty ("percental", ""+percental);
    }

    public void setStartTurn(int newStartTurn)
    {
        if(startTurn==newStartTurn)
            return;

        if(newStartTurn==endTurn)
            throw new InternalError();

        startTurn=newStartTurn;
        fireViewChangedEvent(ViewChangeEvent.C_RANGE_SELECTION);
    }

    public void setEndTurn(int newEndTurn)
    {
        if(endTurn==newEndTurn)
            return;

        if(newEndTurn==startTurn)
            throw new InternalError();

        endTurn=newEndTurn;
        fireViewChangedEvent(ViewChangeEvent.C_RANGE_SELECTION);
    }

    public int getNumOfPlayers()
    {
        return numOfPlayers;
    }

    public int getStartTurn()
    {
        return startTurn;
    }

    public int getEndTurn()
    {
        return endTurn;
    }

    public void setDisplayLabeledInYears(boolean displayYears)
    {
        if(this.displayYears==displayYears)
            return;

        this.displayYears=displayYears;

        CLProperties.setProperty ("showYears", ""+displayYears);
        fireViewChangedEvent(ViewChangeEvent.C_OTHER);
    }

    public boolean isDisplayLabeledInYears()
    {
        return displayYears;
    }

    private void fireViewChangedEvent(int code)
    {
        fireViewChangedEvent(code,-1);
    }

    private void fireViewChangedEvent(int code, int dataType)
    {
        fireViewChangedEvent(code, dataType,false);
    }

    private void fireViewChangedEvent(int code, int dataType,boolean bool)
    {
        ViewChangeEvent evt=new ViewChangeEvent(code, dataType,bool);
        for (Enumeration e = listeners.elements() ; e.hasMoreElements() ;)
        {
            ViewChangeListener current=(ViewChangeListener)e.nextElement();
            current.viewChanged(evt);
        }
    }

    public void addViewChangeListener(ViewChangeListener listener)
    {
        if(listener==null)
            throw new NullPointerException();
        listeners.addElement(listener);
    }
}
