/**
 * @(#)PolyLine.java - used to store an array of x,y points
 *
 * Created on August 1, 2000, 9:05 PM
 */


/**
 *
 * @author  furyhawk <lookmeow@magix.com.sg>
 * @author  Raimar Falke <rf13@inf.tu-dresden.de>
 * @version 0.10
 */

/**
   ********************************************************************
 CivLog - Copyleft (C) 2000 - Wong Teck Meng
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
**********************************************************************
                                                                      */

package org.freeciv.civlog.util;

import org.freeciv.civlog.Debug;
import org.freeciv.civlog.ColorManager;

import java.awt.*;
import java.util.*;

public class PolyLine extends Object
{
    private Vector points=new Vector();
    private int numofPoints = 0;

    private String label = null;
    private Color color = null;

    float maxX, maxY, minX, minY;
    private int maxXIndex, maxYIndex, minXIndex, minYIndex;
    private float averageX, averageY;
    private float totalX=0, totalY=0;
    private int renderedAverageX, renderedAverageY;
    private PolyLineGroup group;

    void setRelativeRange(int point, float startPercent, float endPercent)
    {
        PolyLinePoint p=getPoint(point);
        p.y=endPercent;
        p.startPercent=startPercent;
    }

    public void addPoint (float x, float y)
    {
        points.addElement(new PolyLinePoint(x,y));
        numofPoints++;
    }

    private PolyLinePoint getPoint (int index)
    {
        PolyLinePoint point = (PolyLinePoint)this.points.elementAt (index);
        if (point == null)
            throw new RuntimeException ("PolyLine.getRX(): elementAt("+
                                        index+") empty");
        return point;
    }

    public int getRX (int index)
    {
        return getPoint(index).renderedX;
    }

    public int getRY (int index)
    {
        return getPoint(index).renderedY;
    }

    public int getRStartPercent (int index)
    {
        return getPoint(index).renderedStartPercent;
    }

    public float getX (int index)
    {
        return getPoint(index).x;
    }

    public float getY (int index)
    {
        return getPoint(index).y;
    }

    public void setLabel (String newLabel)
    {
        label = newLabel;
    }

    public void setColor (Color c)
    {
        color = c;
    }

    void startRendering()
    {
        maxX=-Float.MAX_VALUE;
        maxY=-Float.MAX_VALUE;
        minX=Float.MAX_VALUE;
        minY=Float.MAX_VALUE;
        maxXIndex=-1;
        maxYIndex=-1;
        minXIndex=-1;
        minYIndex=-1;

        totalX=0;
        totalY=0;

        for (int i=0; i < numofPoints; i++)
        {
            PolyLinePoint currentPoint=getPoint(i);

            float x=currentPoint.x,y;
            if(group.isRelativeGraph)
                y=currentPoint.y-currentPoint.startPercent;
            else
                y=currentPoint.y;

            if(x>maxX)
            {
                maxX=x;
                maxXIndex=i;
            }

            if(y>maxY)
            {
                maxY=y;
                maxYIndex=i;
            }

            if(x<minX)
            {
                minX=x;
                minXIndex=i;
            }

            if(y<minY)
            {
                minY=y;
                minYIndex=i;
            }
            totalX += currentPoint.x;
            totalY += currentPoint.y;
        }
        float width=maxX-minX;
        float height=maxY-minY;
    }

    void finishRendering(float scaleFactorX,float scaleFactorY,
                         Rectangle outputWindow2)
    {
        for (int i=0; i < numofPoints; i++)
        {
            PolyLinePoint currentPoint=getPoint(i);

            currentPoint.renderedX=Math.round((currentPoint.x-group.minX)*
                                              scaleFactorX)+outputWindow2.x;
            currentPoint.renderedY=outputWindow2.height-
                                   Math.round((currentPoint.y-group.minY)*
                                              scaleFactorY)+outputWindow2.y;
            currentPoint.renderedStartPercent=outputWindow2.height-
                                              Math.round((currentPoint.startPercent-group.minY)*
                                                         scaleFactorY)+outputWindow2.y;

        }
        averageX=totalX/(float)numofPoints;
        averageY=totalY/(float)numofPoints;
        renderedAverageX=Math.round((averageX-group.minX)*scaleFactorX)+
                         outputWindow2.x;
        renderedAverageY=outputWindow2.height-
                         Math.round((averageY-group.minY)*
                                    scaleFactorY)+outputWindow2.y;
    }

    public void drawPolyLine ()
    {
        group.graphics.setColor (color);
        if(!group.isRelativeGraph)
        {
            for (int i=1; i < numofPoints; i++)
            {
                group.graphics.drawLine (getRX (i-1), getRY (i-1),
                                         getRX (i), getRY (i));
            }
        }
        else
        {
            Polygon polygon=new Polygon();

            for (int i=0; i < numofPoints; i++)
                polygon.addPoint(getRX (i), getRY (i));

            for (int i=numofPoints-1; i >=0; i--)
                polygon.addPoint(getRX (i), getRStartPercent (i));

            group.graphics.fillPolygon(polygon);
        }
    }

    /** need to call render() before using this function */
    /** the draw average line */
    public void drawAverage ()
    {

        Graphics2D g2 = (Graphics2D) group.graphics;
        g2.setColor ( ColorManager.getAverageLineColor() );

        //        /** made average line draw using dashed line */
        //        float dash[] = {12.0f};
        //        BasicStroke oldStroke = (BasicStroke) g2.getStroke();
        //        g2.setStroke(new BasicStroke (oldStroke.getLineWidth (),
        //                                      oldStroke.getEndCap (),
        //                                      oldStroke.getLineJoin (),
        //                                      12.0f,
        //                                      dash,
        //                                      oldStroke.getMiterLimit () ));

        g2.drawLine (group.finalOutputWindow.x,
                     renderedAverageY,
                     group.finalOutputWindow.x+
                     group.finalOutputWindow.width,
                     renderedAverageY);

        //        // reset back to old stroke
        //        g2.setStroke (oldStroke);

        g2.setColor ( ColorManager.getAverageLabelColor() ); /** reset back the color */
        g2.drawString ("Avg: " + averageY,
                       group.finalOutputWindow.x+3,
                       renderedAverageY - 3);
    }

    public void drawLabel ()
    {
        int y1;

        group.graphics.setColor (color);

        if(!group.isRelativeGraph)
        {
            group.graphics.drawString ( label,
                                        group.finalOutputWindow.x+
                                        group.finalOutputWindow.width,
                                        getRY (numofPoints-1));
        }
        else
        {
            group.graphics.drawString ( label,
                                        group.finalOutputWindow.x+
                                        group.finalOutputWindow.width,
                                        (getRY (numofPoints-1)+
                                         getRStartPercent (numofPoints-1))/2);
        }
    }

    /** need to calculateMaxMinAvg () before using this function */
    public void drawMaxPoint ()
    {
        if(!group.isRelativeGraph)
        {
            /** use a red dot to mark the max point */
            group.graphics.setColor ( ColorManager.getMaxPointColor() );
            group.graphics.fillOval (getRX (maxYIndex)-2, getRY (maxYIndex)-2,
                                     4, 4);
            group.graphics.setColor (color);
            group.graphics.drawString ( ""+maxY,getRX (maxYIndex)-15,
                                        getRY (maxYIndex)-3);
        }
        else
        {
            group.graphics.setColor ( ColorManager.getMaxLabelColor() );
            group.graphics.drawLine(getRX (maxYIndex),
                                    getRY (maxYIndex),
                                    getRX (maxYIndex),
                                    getRStartPercent (maxYIndex));
            group.graphics.drawString (((int)(maxY*10.0f))/10.0f+"%",
                                       getRX (maxYIndex)+5,
                                       (getRY (maxYIndex)+
                                        getRStartPercent(maxYIndex))/2);
        }
    }

    /** need to calculateMaxMinAvg () before using this function */
    public void drawMinPoint ()
    {
        /** use a red dot to mark the max point */
        group.graphics.setColor ( ColorManager.getMinPointColor() );
        group.graphics.fillOval (getRX (minYIndex)-2, getRY (minYIndex)-2,
                                 4, 4);
        group.graphics.setColor ( ColorManager.getMinLabelColor() );
        /** decide whether to display Min towards the left or right */
        if (getRX (minYIndex) < 40)
        {
            group.graphics.drawString ( "Min " + minY,
                                        getRX (minYIndex)+2,
                                        getRY (minYIndex)-15);
        }
        else
        {
            group.graphics.drawString ( "Min " + minY,
                                        getRX (minYIndex)-40, getRY (minYIndex)-15);
        }
    }

    public boolean isEmpty()
    {
        return (numofPoints==0);
    }

    public void setGroup(PolyLineGroup group)
    {
        this.group=group;
    }

    public int getNoOfPoints()
    {
        return numofPoints;
    }
}
/** End of PolyLine.java */
